import { StreamieQueueError } from './error';
export type BatchedIfConfigured<T, C extends Config> = 'batchSize' extends keyof C ? (C['batchSize'] extends (1 | undefined) ? T : T[]) : T;
export type UnflattenedIfConfigured<T, C extends Config> = 'flatten' extends keyof C ? (C['flatten'] extends true ? T[] : T) : T;
export type OutputIsInputIfFilter<IQT, OQT, C extends Config> = 'isFilter' extends keyof C ? (C['isFilter'] extends true ? IQT : OQT) : OQT;
export type BooleanIfFilter<OQT, C extends Config> = 'isFilter' extends keyof C ? (C['isFilter'] extends true ? boolean : OQT) : OQT;
export type IfFilteredElse<A, B, C extends Config> = 'isFilter' extends keyof C ? (C['isFilter'] extends true ? A : B) : B;
export type Handler<IQT, OQT, C extends Config> = (input: BatchedIfConfigured<IQT, C>, tools: Tools<IQT, OQT, C>) => BooleanIfFilter<UnflattenedIfConfigured<OQT, C>, C> | Promise<BooleanIfFilter<UnflattenedIfConfigured<OQT, C>, C>>;
type Tools<IQT, OQT, C extends Config> = {
    self: Streamie<IQT, OQT, C>;
    push: Streamie<IQT, OQT, C>['push'];
    index: number;
};
export type Config = {
    backpressureAt?: number | {
        input?: number;
        output?: number;
    };
    concurrency?: number;
    batchSize?: number;
    maxBatchWait?: number;
    isFilter?: boolean;
    haltOnError?: boolean;
    flatten?: boolean;
    propagateErrors?: boolean;
};
export type Streamie<IQT extends any, OQT extends any, C extends Config> = {
    push: (item: IQT) => void;
    map: <NOQT extends IfFilteredElse<BatchedIfConfigured<OQT, NC>, any, NC>, NC extends Config>(handler: Handler<OQT, NOQT, NC>, config: NC) => Streamie<OQT, NOQT, NC>;
    filter: <NC extends Omit<Config, 'isFilter'>>(handler: Handler<OQT, boolean, NC>, config: NC) => Streamie<OQT, BatchedIfConfigured<OQT, NC>, NC>;
    pause: (shouldPause?: boolean) => void;
    drain: () => void;
    registerInput: (inputStreamie: Streamie<any, IQT, any>) => void;
    registerOutput: (outputStreamie: Streamie<OQT, any, any>) => void;
    onBackpressureRelease: (eventHandler: () => void) => void;
    onDrained: (eventHandler: () => void) => void;
    onDraining: (eventHandler: () => void) => void;
    onError: (eventHandler: (error: StreamieQueueError<IQT, C>) => void) => void;
    onHalted: (eventHandler: () => void) => void;
    _pushQueueError: (error: StreamieQueueError<any, any>) => void;
    state: {
        backpressure: {
            input: boolean;
            output: boolean;
        };
        isPaused: boolean;
        isDrained: boolean;
        isHalted: boolean;
        count: {
            handling: number;
            started: number;
            queued: {
                input: number;
                output: number;
            };
        };
    };
    promise: Promise<null>;
};
export {};
